from random import uniform

from FGAme import *


class Flappy(Poly):
    """
    A Flappy triangle.

    It jumps and it rotates.
    """

    def __init__(self, **kwds):
        super(Flappy, self).__init__(
            [(0, 0), (40, 0), (20, 80)], color='red', **kwds)
        self.pos = (200, 300)
        self.rotate(uniform(0, 2 * math.pi))
        self.inertia *= 10
        self.omega = uniform(-2, 2)
        self.receiving_input = True

    @listen('key-down', 'space')
    def flappy_up(self):
        """
        Flappy jumps!
        """

        if self.receiving_input:
            self.boost((0, 175))

    @listen('key-down', 'left', delta=0.2)
    @listen('key-down', 'right', delta=-0.2)
    def change_omega(self, delta):
        """
        Change angular velocity
        """

        if self.receiving_input:
            self.omega += delta

    def pre_collision(self, col=None):
        """
        Block user input (the bird is dead).
        """

        self.receiving_input = False


class Game(World):
    """
    Main game class.
    """

    def init(self):
        # Configuration
        self.gravity = 400

        # Obstacles
        self.N = N = 4
        self.obstacles = []
        for i in range(N):
            self.new_obstacle((850 / N) * (i + 1) + 400)

        # Floor and ceiing
        self.floor = self.add.aabb(0, 800, -300, 10, mass='inf', world=self)
        self.ceiling = self.add.aabb(0, 800, 590, 800, mass='inf', world=self)

        # Flappy triangle
        self.flappy = Flappy()
        self.add(self.flappy)

    def new_obstacle(self, pos_x):
        """
        New obstacle in position x.
        """

        size = 50
        speed = 50
        middle = uniform(50 + size, 550 - size)
        lower = self.add.aabb(pos_x, pos_x + 30, 0, middle - size,
                              mass='inf', vel=(-speed, 0))
        upper = self.add.aabb(pos_x, pos_x + 30, middle + size, 600,
                              mass='inf', vel=(-speed, 0), world=self)
        self.obstacles.append([lower, upper])

    def frame_enter_event(self):
        """
        Detects if obstacle had left the screen.
        """

        L = self.obstacles
        if L[0][0].xmax < 0:
            self.remove(L[0][0])
            self.remove(L[0][1])
            del L[0]
            new_x = L[-1][0].xmin + 850 / self.N
            self.new_obstacle(new_x)

        if self.flappy.xmax < -400:
            self.game_over()

    def game_over(self):
        """
        Game over.
        """

        self.stop()

if __name__ == '__main__':
    game = Game()
    game.run()
